@@ -47,6 +47,7 @@
 extern HICON		g_hIconX;
 extern HICON		g_hSmallIconX;
 extern HWND		g_hDlgDepthChange;
+extern Bool             g_fNativeGl;
 
 /*
  * Prototypes for local functions
@@ -64,18 +65,6 @@ winUpdateWindowsWindow (WindowPtr pWin);
 static void
 winFindWindow (pointer value, XID id, pointer cdata);
 
-/*
- * Macros
- */
-
-#define SubSend(pWin) \
-    ((pWin->eventMask|wOtherEventMasks(pWin)) & SubstructureNotifyMask)
-
-#define StrSend(pWin) \
-    ((pWin->eventMask|wOtherEventMasks(pWin)) & StructureNotifyMask)
-
-#define SubStrSend(pWin,pParent) (StrSend(pWin) || SubSend(pParent))
-
 static
 void winInitMultiWindowClass(void)
 {
@@ -86,7 +75,7 @@ void winInitMultiWindowClass(void)
   {
     /* Setup our window class */
     wcx.cbSize=sizeof(WNDCLASSEX);
-    wcx.style = CS_HREDRAW | CS_VREDRAW;
+    wcx.style = CS_HREDRAW | CS_VREDRAW | (g_fNativeGl ? CS_OWNDC : 0);
     wcx.lpfnWndProc = winTopLevelWindowProc;
     wcx.cbClsExtra = 0;
     wcx.cbWndExtra = 0;
@@ -524,21 +513,17 @@ winCreateWindowsWindow (WindowPtr pWin)
   iX = pWin->drawable.x + GetSystemMetrics (SM_XVIRTUALSCREEN);
   iY = pWin->drawable.y + GetSystemMetrics (SM_YVIRTUALSCREEN);
 
-  /* Default positions if none specified */
-  if (!winMultiWindowGetWMNormalHints(pWin, &hints))
-    hints.flags = 0;
-  if ( !(hints.flags & (USPosition|PPosition)) &&
-       !winMultiWindowGetTransientFor (pWin, NULL) &&
-       !pWin->overrideRedirect )
-    {
-      iX = CW_USEDEFAULT;
-      iY = CW_USEDEFAULT;
-    }
-
   iWidth = pWin->drawable.width;
   iHeight = pWin->drawable.height;
 
-    if (winMultiWindowGetTransientFor (pWin, &pDaddy))
+  /* ensure window actually ends up somewhere visible */
+  if (iX > GetSystemMetrics (SM_CXVIRTUALSCREEN))
+    iX = CW_USEDEFAULT;
+
+  if (iY > GetSystemMetrics (SM_CYVIRTUALSCREEN))
+    iY = CW_USEDEFAULT;
+
+  if (winMultiWindowGetTransientFor (pWin, &pDaddy))
     {
       if (pDaddy)
       {
@@ -546,6 +531,18 @@ winCreateWindowsWindow (WindowPtr pWin)
         if (hFore && (pDaddy != (WindowPtr)GetProp(hFore, WIN_WID_PROP))) hFore = NULL;
       }
     }
+  else
+    {
+      /* Default positions if none specified */
+      if (!winMultiWindowGetWMNormalHints(pWin, &hints))
+        hints.flags = 0;
+      if (!(hints.flags & (USPosition|PPosition)) &&
+          !pWin->overrideRedirect)
+      {
+        iX = CW_USEDEFAULT;
+        iY = CW_USEDEFAULT;
+      }
+    }
 
   /* Create the window */
   /* Make it OVERLAPPED in create call since WS_POPUP doesn't support */
@@ -810,7 +807,7 @@ winMinimizeWindow (Window id)
   ErrorF ("winMinimizeWindow\n");
 #endif
 
-  pWin = (WindowPtr) LookupIDByType (id, RT_WINDOW);
+  dixLookupResourceByType((pointer) &pWin, id, RT_WINDOW, NullClient, DixUnknownAccess);
   if (!pWin) 
   { 
       ErrorF("%s: NULL pWin. Leaving\n", __FUNCTION__); 
